/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2006. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.rational.report.web.core.client.internal.ProvisioningClient");

dojo.require("dijit.Dialog");
dojo.require("dojo.cookie");
dojo.require("com.ibm.rational.report.web.core.app.internal.LoginWidget");
dojo.require("com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");

var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var timeoutInterval			= 1800; // default HTTP session timeout in secs
var USERID_COOKIE			= "JazzLoginCookie";

com.ibm.rational.report.web.core.client.internal.ProvisioningClient = {

	getTimeoutInterval: function() {
		return timeoutInterval;
	},
	
	connect: function(url, userid, password, provisionResponseHandler) {
		var connecturl = "connect?url="+url+"&userid="+userid+"&password="+password;
		this._invokeService(connecturl, provisionResponseHandler, arguments);
	},
	
	disconnect: function(provisionResponseHandler) {
		var url = "disconnect";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	installFeature: function(featureId, version, start, provisionResponseHandler) {
		var url = "installFeature?featureId="+featureId+"&version="+version+"&start="+start;
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	startFeature: function(featureId, version, provisionResponseHandler) {
		var url = "startFeature?featureId="+featureId+"&version="+version;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	stopFeature: function(featureId, version, provisionResponseHandler) {
		var url = "stopFeature?featureId="+featureId+"&version="+version;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	startBundle: function(id, provisionResponseHandler) {
		var url = "startBundle?bundleId="+id;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	stopBundle: function(id, provisionResponseHandler) {
		var url = "stopBundle?bundleId="+id;	
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	startAll: function(provisionResponseHandler) {
		var url = "startAll";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	stopAll: function(provisionResponseHandler) {
		var url = "stopAll";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	removeFeature: function(featureId, version, provisionResponseHandler) {
		var url = "removeFeature?featureId="+featureId+"&version="+version;
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	removeAll: function(provisionResponseHandler) {
		var url = "removeAll";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	getStatus: function(core, installed, available, featureId, version, provisionResponseHandler) {
		var url = "status?core="+core+"&installed="+installed+"&available="+available;
		if (featureId && version) {
			url +="&featureId="+featureId+"&version="+version;
		} 
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	isRepositoryAvailable: function(provisionResponseHandler) {
		var url = "isRepositoryAvailable";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	getInstallLog: function(provisionResponseHandler) {
		var url = "status?installLog=true";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
	
	getBundleStatus: function(bundleId, provisionResponseHandler) {
		var url = "status?bundleId="+bundleId;
		this._invokeService(url, provisionResponseHandler, arguments);
	},
		
	requestReset: function(provisionResponseHandler) {
		var url = "requestReset";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
		
	reloadLogSettings: function(provisionResponseHandler) {
		var url = "reloadLogSettings";
		this._invokeService(url, provisionResponseHandler, arguments);
	},
		
	_invokeService: function(url, provisionResponseHandler, args) {
		var self = this;
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var bindArgs = {
			url: dojo.baseUrl + "../../admin/cmd/"+url,
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(response, ioArgs) {
				if (response instanceof Error) {
					var errorObj = {
						status: ioArgs.xhr.status,
						statusText: ioArgs.xhr.statusText
					};
					provisionResponseHandler.doErrorCallback(errorObj, args);
				} 
				else if (ioArgs.xhr.getResponseHeader("X-com-ibm-team-repository-web-auth-msg") == "authrequired") {
					self._loginPrompt(this, provisionResponseHandler, args, "get");
				}
				else {
					provisionResponseHandler.doCallback(response, args);
				}
				
				var timeout = ioArgs.xhr.getResponseHeader("X-com-ibm-team-repository-web-timeout");
				if (typeof timeout != "undefined" && timeout != null && timeout != "") {
					timeoutInterval = parseInt(timeout, 10);
				}
			}
		};
		var xhr = dojo.xhrGet(bindArgs);
	},
	
	_loginPrompt: function(bindArgs, provisionResponseHandler, args, requestMethod, noQueue, errMessage) {
		var workbenchNode = dojo.byId("workbench");
		var dialogWidget = dijit.byId("com-ibm-rational-report-web-LoginWidgetDialog");
		if (dialogWidget == null) {
			var self = this;
			var loginWidget = new com.ibm.rational.report.web.core.app.internal.LoginWidget(null, workbenchNode);
			dialogWidget = new dijit.Dialog({id:"com-ibm-rational-report-web-LoginWidgetDialog",
				execute: dojo.hitch(this, function(dialogArgs) {
					// allow the dialogWidget to be closed
					dialogWidget._alreadyInitialized = true;
					dialogWidget.hide();
					dialogWidget.loginState = "inactive";
					if (dialogArgs.rememberUserID == "on") {
						dojo.cookie(USERID_COOKIE, dialogArgs.userid, { expires: 365 });
					} else {
						dojo.cookie(USERID_COOKIE, "");
					}	
					self._login(dialogArgs.userid, dialogArgs.password, bindArgs, provisionResponseHandler, args, requestMethod);
				})
			}, loginWidget.domNode);
			dialogWidget.loginWidget = loginWidget;
			dialogWidget.loginState = "inactive";
			loginWidget.connect(loginWidget.loginButton, "onclick", function(event) {
				event.preventDefault();
				if (dialogWidget.loginState == "active") {
					dialogWidget.execute(dialogWidget.getValues());
				}
			});
			loginWidget.connect(loginWidget.passwordNode.domNode, "onkeypress", function(event) {
				if (event.keyCode == dojo.keys.ENTER) {
					event.preventDefault();
					if (dialogWidget.loginState == "active") {
						dialogWidget.execute(dialogWidget.getValues());
					}
				}
			});
		}
		if (errMessage != null) {
			dialogWidget.loginWidget.showErrorMessage(errMessage);
		}
		else {
			dialogWidget.loginWidget.hideErrorMessage();
		}
		var storedUserId = dojo.cookie(USERID_COOKIE);
		if (storedUserId) {
			dialogWidget.loginWidget.setUserId(storedUserId);
			dialogWidget.loginWidget.rememberUserIdNode.setChecked(true);
			var i = 1;
		} else {
			var i = 0;
		}
		if (dialogWidget.loginState == "inactive") {
			dialogWidget.show();
			setTimeout(dojo.hitch(dialogWidget, function(){
				dijit.focus(dojo.query('input', this.domNode)[i]); 
			}), 50);
			// prevent the user from closing the dialogWidget without submitting
			dialogWidget._alreadyInitialized = false;
			dialogWidget.loginState = "active";
		}
		if (dialogWidget.queuedRequests == null) {
			dialogWidget.queuedRequests = new Array();
		}
		if (noQueue == null || !noQueue) {
			dialogWidget.queuedRequests.push({
				bindArgs: bindArgs,
				requestMethod: requestMethod,
				responseHandler: provisionResponseHandler,
				args: args
			});
		}
	},

	_login: function(userid, password, bindArgs, provisionResponseHandler, args, requestMethod) {
		var self = this;
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";		
	    var bindArgs= {
	        url: "j_security_check",
	        content: {
	            j_username: userid,
	            j_password: password
	        },
			handleAs: "json",
			headers: headers,
			handle: dojo.hitch(this, function(response, ioArgs) {
				if (response instanceof Error) {
					var errorObj = {
						status: ioArgs.xhr.status,
						statusText: ioArgs.xhr.statusText
					};
					provisionResponseHandler.doErrorCallback(errorObj, args);
				}
				else if (ioArgs.xhr.getResponseHeader("X-com-ibm-team-repository-web-auth-msg") == "authfailed") {
					var dialogWidget = dijit.byId("com-ibm-rational-report-web-LoginWidgetDialog");
					dialogWidget.loginWidget.passwordNode.domNode.value = "";
					self._loginPrompt(bindArgs, provisionResponseHandler, args, requestMethod, true, ServiceabilityMessages.INVALID_USERNAME_PASSWORD);
				}
				else {
					var dialogWidget = dijit.byId("com-ibm-rational-report-web-LoginWidgetDialog");
					if (dialogWidget.queuedRequests != null) {
						while (dialogWidget.queuedRequests.length > 0) {
							var request = dialogWidget.queuedRequests.shift();
							if (request.requestMethod == "post") {						
								xhr = dojo.rawXhrPost(request.bindArgs);
							}
							else {
								xhr = dojo.xhrGet(request.bindArgs);
							}
						}
					}
					
					var arg = {
				    	url: "/jazz/login",
				        content: {
				        	j_username: userid,
				            j_password: password
				        },
						handleAs: "json",
						headers: {
							accept: "text/json"
						},
						preventCache: dojo.isIE,
						handle: dojo.hitch(this, function(response, ioArgs) {							
						})
					};
					dojo.xhrPost(arg);			
				}
	        })
	    };
	    dojo.xhrPost(bindArgs);
	}
};

})();